/* ***********************************************************
 * This file was automatically generated on 2022-05-11.      *
 *                                                           *
 * Rust Bindings Version 2.0.20                              *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

//! Controls one RGB LED.
//!
//! See also the documentation [here](https://www.tinkerforge.com/en/doc/Software/Bricklets/RGBLED_Bricklet_Rust.html).
use crate::{byte_converter::*, converting_receiver::ConvertingReceiver, device::*, ip_connection::GetRequestSender};
pub enum RgbLedBrickletFunction {
    SetRgbValue,
    GetRgbValue,
    GetIdentity,
}
impl From<RgbLedBrickletFunction> for u8 {
    fn from(fun: RgbLedBrickletFunction) -> Self {
        match fun {
            RgbLedBrickletFunction::SetRgbValue => 1,
            RgbLedBrickletFunction::GetRgbValue => 2,
            RgbLedBrickletFunction::GetIdentity => 255,
        }
    }
}

#[derive(Clone, Copy, Debug, Default, PartialEq, Eq, Hash)]
pub struct RgbValue {
    pub r: u8,
    pub g: u8,
    pub b: u8,
}
impl FromByteSlice for RgbValue {
    fn bytes_expected() -> usize { 3 }
    fn from_le_byte_slice(bytes: &[u8]) -> RgbValue {
        RgbValue {
            r: <u8>::from_le_byte_slice(&bytes[0..1]),
            g: <u8>::from_le_byte_slice(&bytes[1..2]),
            b: <u8>::from_le_byte_slice(&bytes[2..3]),
        }
    }
}

#[derive(Clone, Debug, Default, PartialEq, Eq, Hash)]
pub struct Identity {
    pub uid: String,
    pub connected_uid: String,
    pub position: char,
    pub hardware_version: [u8; 3],
    pub firmware_version: [u8; 3],
    pub device_identifier: u16,
}
impl FromByteSlice for Identity {
    fn bytes_expected() -> usize { 25 }
    fn from_le_byte_slice(bytes: &[u8]) -> Identity {
        Identity {
            uid: <String>::from_le_byte_slice(&bytes[0..8]),
            connected_uid: <String>::from_le_byte_slice(&bytes[8..16]),
            position: <char>::from_le_byte_slice(&bytes[16..17]),
            hardware_version: <[u8; 3]>::from_le_byte_slice(&bytes[17..20]),
            firmware_version: <[u8; 3]>::from_le_byte_slice(&bytes[20..23]),
            device_identifier: <u16>::from_le_byte_slice(&bytes[23..25]),
        }
    }
}

/// Controls one RGB LED
#[derive(Clone)]
pub struct RgbLedBricklet {
    device: Device,
}
impl RgbLedBricklet {
    pub const DEVICE_IDENTIFIER: u16 = 271;
    pub const DEVICE_DISPLAY_NAME: &'static str = "RGB LED Bricklet";
    /// Creates an object with the unique device ID `uid`. This object can then be used after the IP Connection `ip_connection` is connected.
    pub fn new<T: GetRequestSender>(uid: &str, req_sender: T) -> RgbLedBricklet {
        let mut result = RgbLedBricklet { device: Device::new([2, 0, 0], uid, req_sender, 0) };
        result.device.response_expected[u8::from(RgbLedBrickletFunction::SetRgbValue) as usize] = ResponseExpectedFlag::False;
        result.device.response_expected[u8::from(RgbLedBrickletFunction::GetRgbValue) as usize] = ResponseExpectedFlag::AlwaysTrue;
        result.device.response_expected[u8::from(RgbLedBrickletFunction::GetIdentity) as usize] = ResponseExpectedFlag::AlwaysTrue;
        result
    }

    /// Returns the response expected flag for the function specified by the function ID parameter.
    /// It is true if the function is expected to send a response, false otherwise.
    ///
    /// For getter functions this is enabled by default and cannot be disabled, because those
    /// functions will always send a response. For callback configuration functions it is enabled
    /// by default too, but can be disabled by [`set_response_expected`](crate::rgb_led_bricklet::RgbLedBricklet::set_response_expected).
    /// For setter functions it is disabled by default and can be enabled.
    ///
    /// Enabling the response expected flag for a setter function allows to detect timeouts
    /// and other error conditions calls of this setter as well. The device will then send a response
    /// for this purpose. If this flag is disabled for a setter function then no response is sent
    /// and errors are silently ignored, because they cannot be detected.
    ///
    /// See [`set_response_expected`](crate::rgb_led_bricklet::RgbLedBricklet::set_response_expected) for the list of function ID constants available for this function.
    pub fn get_response_expected(&mut self, fun: RgbLedBrickletFunction) -> Result<bool, GetResponseExpectedError> {
        self.device.get_response_expected(u8::from(fun))
    }

    /// Changes the response expected flag of the function specified by the function ID parameter.
    /// This flag can only be changed for setter (default value: false) and callback configuration
    /// functions (default value: true). For getter functions it is always enabled.
    ///
    /// Enabling the response expected flag for a setter function allows to detect timeouts and
    /// other error conditions calls of this setter as well. The device will then send a response
    /// for this purpose. If this flag is disabled for a setter function then no response is sent
    /// and errors are silently ignored, because they cannot be detected.
    pub fn set_response_expected(&mut self, fun: RgbLedBrickletFunction, response_expected: bool) -> Result<(), SetResponseExpectedError> {
        self.device.set_response_expected(u8::from(fun), response_expected)
    }

    /// Changes the response expected flag for all setter and callback configuration functions of this device at once.
    pub fn set_response_expected_all(&mut self, response_expected: bool) { self.device.set_response_expected_all(response_expected) }

    /// Returns the version of the API definition (major, minor, revision) implemented by this API bindings.
    /// This is neither the release version of this API bindings nor does it tell you anything about the represented Brick or Bricklet.
    pub fn get_api_version(&self) -> [u8; 3] { self.device.api_version }

    /// Sets the *r*, *g* and *b* values for the LED.
    pub fn set_rgb_value(&self, r: u8, g: u8, b: u8) -> ConvertingReceiver<()> {
        let mut payload = vec![0; 3];
        payload[0..1].copy_from_slice(&<u8>::to_le_byte_vec(r));
        payload[1..2].copy_from_slice(&<u8>::to_le_byte_vec(g));
        payload[2..3].copy_from_slice(&<u8>::to_le_byte_vec(b));

        self.device.set(u8::from(RgbLedBrickletFunction::SetRgbValue), payload)
    }

    /// Returns the *r*, *g* and *b* values of the LED as set by [`set_rgb_value`].
    pub fn get_rgb_value(&self) -> ConvertingReceiver<RgbValue> {
        let payload = vec![0; 0];

        self.device.get(u8::from(RgbLedBrickletFunction::GetRgbValue), payload)
    }

    /// Returns the UID, the UID where the Bricklet is connected to,
    /// the position, the hardware and firmware version as well as the
    /// device identifier.
    ///
    /// The position can be 'a', 'b', 'c', 'd', 'e', 'f', 'g' or 'h' (Bricklet Port).
    /// A Bricklet connected to an [Isolator Bricklet](isolator_bricklet) is always at
    /// position 'z'.
    ///
    /// The device identifier numbers can be found [here](device_identifier).
    /// |device_identifier_constant|
    pub fn get_identity(&self) -> ConvertingReceiver<Identity> {
        let payload = vec![0; 0];

        self.device.get(u8::from(RgbLedBrickletFunction::GetIdentity), payload)
    }
}
